var AppFeed = (function () {
  // elements
  var $appFeed = null;
  var $lnkViewFeed = null;
  var $lnkToggleFeed = null;
  var $feedContainer = null;

  var feedCopy = [];
  var serverOffset = null;
  var currentlyShown;

  var Service = null;

  var localStorageKey = 'feedLastSeen';
  var dropdownHandler = '';
  var apiServerOffset = '';

  function init() {
    Service = Services(
      PBConstants.BASE_API_URL,
      PBConstants.MARKET_CAP_API_URL,
      PBConstants.CURRENCIES_API_URL,
      PBConstants.GLOBAL_MARKET_CAP_API_URL
    );
    dropdownHandler = DropdownHandler();
    $appFeed = $('#appFeed');
    $lnkViewFeed = $('#lnkViewFeed');
    $lnkToggleFeed = $('#lnkToggleFeed');
    $feedContainer = $('#newsFeedContainer .dropdown-menu');
    // show default # of nes
    currentlyShown = dropdownHandler.toggleShown(PBConstants.FEED.MAX, PBConstants.FEED);

    // show empty news.
    dropdownHandler.showMessage(i18next.t('constantMsgs.loading'), $appFeed, $lnkViewFeed);
    // Add the events
    addEvents();
  }

  /**
   * Display news in dropdown.
   * @param {*} feedList
   * @param {*} timezoneOffset
   */
  function display(feedList, timezoneOffset) {
    serverOffset = timezoneOffset;

    if (!Array.isArray(feedList)) {
      feedList = [];
    }

    // Generate default news html.
    showFeed(
      feedList.slice(0, currentlyShown.count),
      serverOffset
    );
    $lnkViewFeed.text(currentlyShown.text);

    // keep a copy of the first X news for "View all"
    feedCopy = feedList.slice(0, PBConstants.FEED.MAX);
  }

  function addEvents() {
    $lnkViewFeed.click(function () {
      currentlyShown = dropdownHandler.toggleShown(currentlyShown.count, PBConstants.FEED);
      showFeed(
        feedCopy.slice(0, currentlyShown.count),
        serverOffset
      );
      $lnkViewFeed.text(currentlyShown.text);
      return false;
    });

    $lnkToggleFeed.click(function (event) {
      event.preventDefault();
      displayUnseenCount(0);
      if (feedCopy.length > 0) {
        dropdownHandler.setLastSeen(feedCopy, 'date_published', localStorageKey);
      }
    });

    // On click on dropdown item redirect to news page and show clicked news.
    $appFeed.on('click', '.news-dropdown-link', function (e) {
      e.preventDefault();
      var id = $(this).data('id');
      if (id) {
        localStorage.setItem('currentNews', id);
      }
      var href = $(this).attr('href');
      page(href);
    });
  }

  function showFeed(feed, serverOffset) {
    $appFeed.empty();
    apiServerOffset = serverOffset;
    if (!feed || feed.length === 0) {
      // Adding no-notification class when news length is zero.
      $feedContainer.addClass('no-notification');
      // no news found, show empty and hide the "view all / view less" button
      dropdownHandler.showMessage(i18next.t('constantMsgs.noFeed'), $appFeed, $lnkViewFeed);
      return;
    }
    // Removing class no-notification if news length is not zero.
    if ($feedContainer.hasClass('no-notification')) {
      $feedContainer.removeClass('no-notification');
    }

    // news found, show the "view all / view less" button.
    $lnkViewFeed.show();

    var lastSeen = dropdownHandler.getLastSeen(localStorageKey);
    var lastSeenDt = new Date(lastSeen);
    var unseenNews = 0;

    var newsHtml = '';
    for (var i = 0; i !== feed.length; ++i) {
      var currentNews = feed[i];
      var currentDate = new Date();
      var date = new Date(
        DateTimeHelper.getUTCSuffixDate(currentNews.date_published)
      );
      const newsModifiedDate = new Date(currentNews.date_modified);

      // To consider a news only unseen when current news is not older than 5 days.
      // FIXME: Get from time from constants.
      if ((currentDate.getTime() - newsModifiedDate.getTime() <= 432000000)) {
        ++unseenNews;
      }
      var time = DateTimeHelper.getDateAndTimeAgoForNotifications(
        currentNews.date_published,
        serverOffset
      );
      time = time[0] + ' (' + time[1] + ')';
      var content = currentNews.content_text.slice(0, PBConstants.FEED.NUMBER_OF_CONTENT_CHAR) + '...';
      newsHtml += getFeedHtml(
        currentNews.title,
        content,
        time,
        i
      );
    }

    $appFeed.append(newsHtml);

    // update the last seen value.
    displayUnseenCount(unseenNews);
  }

  function getFeedHtml(title, content, time, iteration) {
    return '<a href="news" class="news-dropdown-link" data-id="news-' + iteration + '"><div class="dropdown-item notify-item">' +
      '<p class="notify-details" >' +
      '<span class="title">' + title + '</span>' +
      '<span class="content text-muted">' + content + '</span>' +
      '<small class="text-muted">' +
      time + '</small>' +
      '</p></div></a>';
  }


  function displayUnseenCount(count) {
    if ($appFeed.is(':visible')) {
      // news dropdown is visible nothing to update, so the latest notification
      // must have already been seen ... the last seen.
      dropdownHandler.setLastSeen(feedCopy, 'date_published', localStorageKey);
      return;
    }
    $lnkToggleFeed.find('span.badge').hide().end().parent().removeClass('bounce');
    if (count === 0) {
      return;
    }
    $lnkToggleFeed
      .find('span.badge')
      .html(count)
      .show().end().parent().addClass('bounce');
  }

  function callFeed(data) {
    // If feed call fails then ignore and show no news string.
    Service.getFeed(PBConstants.API_FEED).done(function (response) {
      dropdownHandler.showMessage(i18next.t('constantMsgs.loading'), $appFeed, $lnkViewFeed);
      if (response && response.items) {
        display(response.items, data.timeZoneOffset);
        displayNewsPageNews();
      } else {
        dropdownHandler.showMessage(i18next.t('constantMsgs.noFeed'), $appFeed, $lnkViewFeed);
      }
    });
    // Call feed for every x seconds.
    setTimeout(callFeed, PBConstants.FEED_REFRESH, data);
  }

  function displayNewsPageNews() {
    if (window.location.href.indexOf('news') === -1) {
      return;
    }
    var feedList = feedCopy;
    var feedHtml = '';
    if (!feedList.length) {
      $('#newsCotnainer').html(`<h2>${i18next.t('constantMsgs.noFeed')}</h2>`);
    }
    for (var i = 0; i < feedList.length; i++) {
      var currentNews = feedList[i];
      var date = DateTimeHelper.getDateAndTimeAgoForNotifications(
        currentNews.date_published,
        apiServerOffset
      );
      feedHtml += '<div class="row news-row" id="news-' + i + '">' +
        '<div class="col-lg-12 col-md-12">' +
        '<div class="widget-bg-color-icon card-box fadeInDown animated">' +
        '<span class="feed-posted-on">' + i18next.t('news.postedOn') + ' ' + date[0] + '</span>' +
        '<a target="_blank" href="' + currentNews.url + '"><h2 class="feed-title">' + currentNews.title + '</h2></a>' +
        '<div class="feed-content">' +
        currentNews.content_html +
        '</div></div></div></div>';
    }
    $('#newsCotnainer').html(feedHtml);
    // Scroll to selected news.
    var currentId = localStorage.getItem('currentNews');
    var $clickedNewsElement = currentId && $('#' + currentId) ? $('#' + currentId) : '';
    localStorage.removeItem('currentNews');
    var $scrollableParentElement = $('body, html');
    // In lower resolutions body is scrollable, in higher resolutions div inside body is scrollable.
    if (window.innerWidth > PBConstants.MIN_RESOLUTION_FOR_FIXED_HEADERS) {
      $scrollableParentElement = $('#contentPage');
    }
    if ($clickedNewsElement && $clickedNewsElement.length) {
      $scrollableParentElement.animate({
        scrollTop: $clickedNewsElement.position().top
      },
        'slow'
      );
    }
  }

  return {
    display: display,
    init: init,
    callFeed: callFeed,
    displayNewsPageNews: displayNewsPageNews
  };
})();
