const monthNames = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"];

// Caching selectors
var $dtDailyProfit = $('#dtDailyProfit');

var dailyProfitTable = null;
var serverData = null;
var isDataTable = false;
var marketCapCurrencies = JSON.parse(localStorage.getItem(NormalizerService.generateLocalStorageKey('currencies')))

function proInit() {
    jQuery.post(THE_BASE_URL + '/monitoring/data.json', function (response) {
        serverData = response;
        var currency = response.settings.currency
        var market = response.market
        var currencyPrice = marketCapCurrencies[currency] || 1

        var marketPrice = getMarketPrice(market, response) // get the market price from the api data

        var tcv = response.pairsBalance + response.totalPairsCurrentValue  + response.totalDCACurrentValue + response.totalPendingCurrentValue + response.totalDustCurrentValue

        currentValueDistChart(response.pairsBalance, response.totalPairsCurrentValue, response.totalDCACurrentValue, response.totalPendingCurrentValue)
        dailyCummulativeChart(response.proStats.dailyStats)

        formatProfitablePairsData(response.sellLogData)

		$('.currencyField').html(currency);
        $('.marketLabel').html(market);

        profitFields(response.stats, marketPrice, tcv, market, currency, currencyPrice)
        monthlyProfits(response.proStats.monthlyStats, marketPrice, currency, currencyPrice)
        currentHoldingTable(serverData.gainLogData, serverData.pendingLogData, serverData.dcaLogData, serverData.dustLogData, marketPrice, currency, currencyPrice)

        getSparklineData(currency,market)

        // To only render daily profits datatable when current tab is Pro Tools.
        if (window.location.href.includes('pro-tool'))  {
            data = response.proStats.dailyStats.reverse();
            renderDailyProfitsTable(data);
        }

        dcaCalculation()
    });
}

// Adding event listeners on page load for pro-tools.
addEventListenerForProTools();

function getMarketPrice(market, response) {
    var marketPrice = 0

    switch(market) {
        case 'ETH':
            marketPrice = response.ETHUSDTPrice
            break;
        case 'TUSD':
            marketPrice = response.TUSDUSDTPrice
            break;
        case 'PAX':
            marketPrice = response.PAXUSDTPrice
            break;
        case 'BNB':
            marketPrice = response.BNBUSDTPrice
            break;
        case 'XRP':
            marketPrice = response.XRPUSDTPrice
            break;
        case 'USDC':
            marketPrice = response.USDCUSDTPrice
            break;
        case 'HT':
            marketPrice = response.HTUSDTPrice
            break;
        case 'USDS':
            marketPrice = response.USDSUSDTPrice
            break;
        case 'NEO':
            marketPrice = response.NEOUSDTPrice
            break;
        case 'KCS':
            marketPrice = response.KCSUSDTPrice
            break;
        case 'XMR':
            marketPrice = response.XMRUSDTPrice
            break;
        case 'USDT':
            marketPrice = 1
            break;
        case 'USD':
            marketPrice = 1
            break;
        case 'HUSD':
            marketPrice = 1
            break;
        case 'DAI':
            marketPrice = 1
            break;
        default:
            marketPrice = response.BTCUSDTPrice
      }

    return marketPrice
}

function dcaTableSwitch(value) {
    if(value == "anderson") {
        $("#andersonTable").parent().show()
        $("#triggerTable").parent().hide()
        $("#grantTable").parent().hide()
        $("#dcaExp").parent().show()
        $("#grantCoinTable").parent().hide()
        $("#coins").parent().parent().parent().hide()
        $("#triggers").parent().parent().parent().hide()
        dcaCalculation()
    } else if(value == "trigger") {
        $("#andersonTable").parent().hide()
        $("#triggerTable").parent().show()
        $("#grantTable").parent().hide()
        $("#dcaExp").parent().show()
        $("#grantCoinTable").parent().hide()
        $("#coins").parent().parent().parent().hide()
        $("#triggers").parent().parent().parent().show()
        dcaCalculation()
    } else if(value == "grant") {
        $("#andersonTable").parent().hide()
        $("#triggerTable").parent().hide()
        $("#grantTable").parent().show()
        $("#dcaExp").parent().hide()
        $("#grantCoinTable").parent().show()
        $("#coins").parent().parent().parent().show()
        $("#triggers").parent().parent().parent().hide()
        dcaCalculation()
    }


}

function dcaCalculation() {
    var calcType = $("#calcType").val();
    var startBalance = parseFloat($("#startBalance").val());
    var pairsToTrade = parseInt($("#pairsToTrade").val());
    var dcaLevels = parseInt($("#dcaLevels").val());
    var maxCost = parseFloat($("#maxCost").val());
    var maxCostPercentage = parseFloat($("#maxCostPercentage").val());
    var usePercentage = $("#usePercentage").val();
    var coins = $("#coins").val();

    if(calcType == "anderson") {
        var dca = [];

        var maxCostA = 0;
        if (usePercentage == "true") {
            maxCostA = (startBalance * maxCostPercentage) / 100;
        } else {
            maxCostA = maxCost;
        }

        var costCoin = Math.round10(maxCostA * Math.pow(2, dcaLevels), -8)
        var balanceNeeded = Math.round10(costCoin * pairsToTrade, -8);
        var extendedValue = Math.round10(startBalance - balanceNeeded, -8);
        var extendedPercent = Math.round10(extendedValue / startBalance * 100, -2);

        $("#balanceNeeded").text(balanceNeeded).removeClass("text-success text-danger").addClass(balanceNeeded < startBalance ? 'text-success' : 'text-danger');
        $("#extendedValue").text(extendedValue).removeClass("text-success text-danger").addClass(balanceNeeded < startBalance ? 'text-success' : 'text-danger');
        $("#extendedPercent").text(extendedPercent + "%").removeClass("text-success text-danger").addClass(balanceNeeded < startBalance ? 'text-success' : 'text-danger');

        for (var i = 1; i <= 10; i++) {
            dca[i - 1] = [];
            for (var j = 0; j <= 10; j++) {
                var costPerPair = Math.round10(maxCostA * Math.pow(2, j), -8);
                dca[i - 1][j] = Math.round10(costPerPair * i, -8);
            }
        }

        const tBody = $("#andersonTBody");
        $(tBody).empty();

        $.each(dca, function (key, item) {
            const tr = $("<tr></tr>")
                .append($("<td></td>").text("Pair " + (key + 1)))
                .append($("<td></td>").text(item[1]).removeClass("text-success text-danger").addClass(startBalance > item[1] ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text(item[2]).removeClass("text-success text-danger").addClass(startBalance > item[2] ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text(item[3]).removeClass("text-success text-danger").addClass(startBalance > item[3] ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text(item[4]).removeClass("text-success text-danger").addClass(startBalance > item[4] ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text(item[5]).removeClass("text-success text-danger").addClass(startBalance > item[5] ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text(item[6]).removeClass("text-success text-danger").addClass(startBalance > item[6] ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text(item[7]).removeClass("text-success text-danger").addClass(startBalance > item[7] ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text(item[8]).removeClass("text-success text-danger").addClass(startBalance > item[8] ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text(item[9]).removeClass("text-success text-danger").addClass(startBalance > item[9] ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text(item[10]).removeClass("text-success text-danger").addClass(startBalance > item[10] ? 'text-success' : 'text-danger'));

            tr.appendTo(tBody);
        })
    } else if(calcType == "trigger") {
        var triggerDca = [];
        var triggers = $('#triggers').val().split(",");

        var maxCostA = 0;
        if (usePercentage == "true") {
            maxCostA = (startBalance * maxCostPercentage) / 100;
        } else {
            maxCostA = maxCost;
        }

        for (var i = 0; i < triggers.length; i++) {
            triggerDca[i] = [];

            for(var j = 0;j < 10;j++) {
                if(i === 0) {
                    triggerDca[i][j] = (((((triggers[i] / 100) * maxCostA) + maxCostA) * (j+1)))
                } else {
                    var pair = triggerDca[i-1]
                    triggerDca[i][j] = (((triggers[i] / 100) * parseFloat(pair[j])) + parseFloat(pair[j]))
                }
            }
        }

        balanceNeeded = Math.round10(triggerDca[dcaLevels - 1][pairsToTrade - 1], -8)
        extendedValue = Math.round10(startBalance - balanceNeeded, -8)
        extendedPercent = Math.round10(extendedValue / startBalance * 100, -2)

        $("#balanceNeeded").text(balanceNeeded).removeClass("text-success text-danger").addClass(balanceNeeded < startBalance ? 'text-success' : 'text-danger');
        $("#extendedValue").text(extendedValue).removeClass("text-success text-danger").addClass(balanceNeeded < startBalance ? 'text-success' : 'text-danger');
        $("#extendedPercent").text(extendedPercent + "%").removeClass("text-success text-danger").addClass(balanceNeeded < startBalance ? 'text-success' : 'text-danger');

        const tBody = $("#trigerTBody");
        $(tBody).empty();

        $.each(triggerDca, function (key, item) {

            const tr = $("<tr></tr>")
                .append($("<td></td>").text(("Trigger " + (key + 1) + ": " + triggers[key] + "%")))
                .append($("<td></td>").text((item[0]).toFixed(3)).removeClass("text-success text-danger").addClass(startBalance > item[0] ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text((item[1]).toFixed(3)).removeClass("text-success text-danger").addClass(startBalance > item[1] ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text((item[2]).toFixed(3)).removeClass("text-success text-danger").addClass(startBalance > item[2] ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text((item[3]).toFixed(3)).removeClass("text-success text-danger").addClass(startBalance > item[3] ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text((item[4]).toFixed(3)).removeClass("text-success text-danger").addClass(startBalance > item[4] ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text((item[5]).toFixed(3)).removeClass("text-success text-danger").addClass(startBalance > item[5] ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text((item[6]).toFixed(3)).removeClass("text-success text-danger").addClass(startBalance > item[6] ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text((item[7]).toFixed(3)).removeClass("text-success text-danger").addClass(startBalance > item[7] ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text((item[8]).toFixed(3)).removeClass("text-success text-danger").addClass(startBalance > item[8] ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text((item[9]).toFixed(3)).removeClass("text-success text-danger").addClass(startBalance > item[9] ? 'text-success' : 'text-danger'));

            tr.appendTo(tBody);
        })



    } else if(calcType == "grant") {
        var grantDca = [];
        var grantCoinDca = [];

        var maxCostA = 0;
        if (usePercentage == "true") {
            maxCostA = (startBalance * maxCostPercentage) / 100;
        } else {
            maxCostA = maxCost;
        }

        for (var i = 1; i <= 30; i++) {
            grantDca[i - 1] = [];
            for (var j = 0; j <= 10; j++) {
                if(j + 1 > pairsToTrade) {
                    grantDca[i - 1][j] = 0;
                } else {
                    var costPerPair = maxCost;
                    grantDca[i - 1][j] = Math.round10(costPerPair * i, -8);
                }

            }
        }

        const tBody = $("#grantTBody");
        $(tBody).empty();

        $.each(grantDca, function (key, item) {
            var sum = 0;
            $.each(item,function(){sum+=parseFloat(this) || 0;});

            const tr = $("<tr></tr>")
                .append($("<td></td>").text((key + 1)))
                .append($("<td></td>").text(item[0] > 0 ? item[0] : '' ).removeClass("text-danger text-success").addClass(grantHighlight(key + 1, 1, maxCost) ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text(item[1] > 0 ? item[1] : '' ).removeClass("text-danger text-success").addClass(grantHighlight(key + 1, 2, maxCost) ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text(item[2] > 0 ? item[2] : '' ).removeClass("text-danger text-success").addClass(grantHighlight(key + 1, 3, maxCost) ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text(item[3] > 0 ? item[3] : '' ).removeClass("text-danger text-success").addClass(grantHighlight(key + 1, 4, maxCost) ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text(item[4] > 0 ? item[4] : '' ).removeClass("text-danger text-success").addClass(grantHighlight(key + 1, 5, maxCost) ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text(item[5] > 0 ? item[5] : '' ).removeClass("text-danger text-success").addClass(grantHighlight(key + 1, 6, maxCost) ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text(item[6] > 0 ? item[6] : '' ).removeClass("text-danger text-success").addClass(grantHighlight(key + 1, 7, maxCost) ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text(item[7] > 0 ? item[7] : '' ).removeClass("text-danger text-success").addClass(grantHighlight(key + 1, 8, maxCost) ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text(item[8] > 0 ? item[8] : '' ).removeClass("text-danger text-success").addClass(grantHighlight(key + 1, 9, maxCost) ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text(item[9] > 0 ? item[9] : '' ).removeClass("text-danger text-success").addClass(grantHighlight(key + 1, 10, maxCost) ? 'text-success' : 'text-danger'))
                .append($("<td></td>").text((sum).toFixed(6)));

            tr.appendTo(tBody);
        })

        for (var i = 1; i <= 30; i++) {
            grantCoinDca[i - 1] = [];
            for (var j = 0; j <= 10; j++) {
                var costPerPair = maxCost;
                grantCoinDca[i - 1][j] = Math.round10(costPerPair * i, -8);
            }
        }

        const tBodyGrant = $("#grantCoinTableTBody");
        $(tBodyGrant).empty();

        for(z = 0;z< 30;z++) {
            const trGrant = $("<tr></tr>")
                .append($("<td></td>").text((z + 1)))
                .append($("<td></td>").text((maxCost * (z + 1)).toFixed(6)))
                .append($("<td></td>").text(((maxCost / (maxCost * (z + 1))) * 100).toFixed(2) + "%"))
                .append($("<td></td>").text(coins))
                .append($("<td></td>").text(coins * (z + 1)))
                .append($("<td></td>").text(z == 0 ? maxCost * (z + 1) : grantTable(z+1, maxCost)))

            trGrant.appendTo(tBodyGrant)
        }

    } else {

    }
}

function grantTable(number, maxCost) {
    var cost = 0

    for (var i = 1; i <= number; i++) {
      cost += maxCost * i
    }

    return parseFloat(cost.toFixed(6))
}

function grantHighlight(dca, pair, maxCost) {
    var cost = grantTable(dca, maxCost)
    var combinedCost = cost * pair

    var startBalance = $("#startBalance").val()
    if (combinedCost > startBalance) {
      return false
    }

    return true
}

function dcaSwitchCost(val) {
    if(val == "maxCostPercentage") {
        $("#usePercentage").val("true")
        $("#maxCost").parent().parent().parent().parent().hide();
        $("#maxCostPercentage").parent().parent().parent().parent().show();
    } else {
        $("#usePercentage").val("false")
        $("#maxCostPercentage").parent().parent().parent().parent().hide();
        $("#maxCost").parent().parent().parent().parent().show();
    }
}

function formatProfitablePairsData(data){
  var group = _.groupBy(data, 'currency')

  var result = _.map(_.keys(group), function(e) {
    return _.reduce(group[e], function(r, o) {

        var currentPrice = o.soldAmount * o.currentPrice
        var currentFeeValue = currentPrice * (o.fee / 100)
        var currentPriceWithoutFees = currentPrice - currentFeeValue

        var profit = currentPriceWithoutFees - o.totalCost

        return r.profit += +profit, r
            }, {
                currency: e,
                profit: 0,
                sum: group[e].length
            })
  })

    var mpObject = _.orderBy(result, ['profit','currency'], ['desc'])
    mpLabels = []
    mpData = []

    var lpObject = _.orderBy(result, ['profit','currency'], ['asc'])
    lpLabels = []
    lpData = []

    var msObject = _.orderBy(result, ['sum','currency'], ['desc'])
    msLabels = []
    msData = []

    for(i = 0; i < mpObject.length ;i++) {
		if(i < 10){
			mpLabels.push(mpObject[i].currency)
			mpData.push(mpObject[i].profit)

			lpLabels.push(lpObject[i].currency)
			lpData.push(lpObject[i].profit)

			msLabels.push(msObject[i].currency)
			msData.push(msObject[i].sum)
		}
    }

    updateProfitablePairsChart();

}

function updateProfitablePairsChart(){

  $('#mppChart').remove();
  $('#mppChartContainer').append('<canvas id="mppChart"></canvas>');
  var activeId = $(".ppTab-active").attr('id')

  if(activeId == 'mpTab'){
    profitablePairsChart(mpLabels,mpData)
  }else if(activeId == 'lpTab'){
    profitablePairsChart(lpLabels,lpData)
  }else if(activeId == 'msTab'){
    profitablePairsChart(msLabels,msData)
  }
}

function monthlyProfits(monthly, marketPrice, currency, currencyPrice) {
    const tBody = $("#monthlySalesTable");
    $(tBody).empty();

    $.each(monthly, function (key, item) {
        var res = item.month.split("-")
        var month = monthNames[res[0] - 1]

        var profitColor = item.totalProfitCurrency > 0 ? "text-success" : item.totalProfitCurrency < 0 ? "text-danger" : "grey"
        var percentageColor = item.avgProfit > 0 ? "text-success" : item.avgProfit < 0 ? "text-danger" : "grey"

        var price = (((item.totalProfitCurrency).toFixed(8) * marketPrice) * currencyPrice).toFixed(2)

        const tr = $("<tr></tr>")
            .append($("<td class='blue'></td>").text(month + " " + res[1]))
            .append($("<td></td>").text(item.totalSales))
            .append($("<td class='"+profitColor+"'></td>").text((item.totalProfitCurrency).toFixed(8) + " (" + currency + " " + price + ")"))
            .append($("<td class='"+percentageColor+"'></td>").text((item.avgProfit || 0).toFixed(2) + "%"));

        tr.appendTo(tBody);
    })

}

function currentHoldingTable(pairs, pending, dca, dust, marketPrice, currency, currencyPrice) {
    pairsAmount = pairs.length
    pairsCost = 0
    pairsValue = 0
    pairsProfit = 0
    // pendingData, dcaData, DustData
    for (i = 0; i < pairsAmount; i++) {
        pairsCost = parseFloat(pairsCost) + parseFloat(pairs[i].totalCost)

        var fee = pairs[i].fee / 100
        var amount = (pairs[i].totalAmount)
        var currentPrice = amount * pairs[i].currentPrice
        var feeValue = currentPrice * fee

        pairsValue = parseFloat(pairsValue) + parseFloat(currentPrice - feeValue)
    }

    pairsAmount = parseFloat(pairsAmount)
    pairsCost = parseFloat(pairsCost).toFixed(6)
    pairsValue = parseFloat(pairsValue).toFixed(6)

    pairsCostCurrency = (pairsCost * marketPrice) * currencyPrice
    pairsValueCurrency = (pairsValue * marketPrice) * currencyPrice

    document.getElementById("chPairsAmount").innerHTML = pairsAmount
    document.getElementById("chPairsCost").innerHTML = pairsCost + " <small>(" + currency + " " + (pairsCostCurrency).toFixed(2) + ")</small>"
    document.getElementById("chPairsValue").innerHTML = pairsValue + " <small>(" + currency + " " + (pairsValueCurrency).toFixed(2) + ")</small>"

    pairsProfitPercentage = ((pairsValue - pairsCost) / pairsCost) * 100 || 0
    pairsProfitCurrency = ((pairsValue - pairsCost) * marketPrice) * currencyPrice
    pairsElement = document.getElementById("chPairsProfit")
    pairsElement.innerHTML = parseFloat(pairsProfitPercentage).toFixed(2) + "% <small>(" + currency + " " + (pairsProfitCurrency).toFixed(2) + ")</small>"
    pairsElement.classList.remove("gray")
    pairsElement.classList.add(pairsProfitPercentage > 0 ? "text-success" : pairsProfitPercentage < 0 ? "text-danger" : "gray")

    pendingAmount = pending.length
    pendingCost = parseFloat(0)
    pendingValue = 0
    pendingProfit = 0

    for (i = 0; i < pendingAmount; i++) {
        pendingCost = parseFloat(pendingCost) + parseFloat(pending[i].totalCost)
        var fee = pending[i].fee / 100
        var amount = (pending[i].totalAmount)
        var currentPrice = amount * pending[i].currentPrice
        var feeValue = currentPrice * fee

        pendingValue = parseFloat(pendingValue) + parseFloat(currentPrice - feeValue)
    }

    pendingAmount = parseFloat(pendingAmount)
    pendingCost = parseFloat(pendingCost).toFixed(6)
    pendingValue = parseFloat(pendingValue).toFixed(6)

    pendingCostCurrency = (pendingCost * marketPrice) * currencyPrice
    pendingValueCurrency = (pendingValue * marketPrice) * currencyPrice

    document.getElementById("chPendingAmount").innerHTML = pendingAmount
    document.getElementById("chPendingCost").innerHTML = pendingCost + " <small>(" + currency + " " + (pendingCostCurrency).toFixed(2) + ")</small>"
    document.getElementById("chPendingValue").innerHTML = pendingValue + " <small>(" + currency + " " + (pendingValueCurrency).toFixed(2) + ")</small>"

    pendingProfitPercentage = ((pendingValue - pendingCost) / pendingCost) * 100 || 0
    pendingProfitCurrency = ((pendingValue - pendingCost) * marketPrice) * currencyPrice
    pendingElement = document.getElementById("chPendingProfit")
    pendingElement.innerHTML = (pendingProfitPercentage).toFixed(2) + "% <small>(" + currency + " " + (pendingProfitCurrency).toFixed(2) + ")</small>"
    pendingElement.classList.remove("gray")
    pendingElement.classList.add(pendingProfitPercentage > 0 ? "text-success" : pendingProfitPercentage < 0 ? "text-danger" : "gray")

    dcaAmount = dca.length
    dcaCost = 0
    dcaValue = 0
    dcaProfit = 0
    for (i = 0; i < dcaAmount; i++) {
        dcaCost = parseFloat(dcaCost) + parseFloat(dca[i].totalCost)
        var fee = dca[i].fee / 100
        var amount = (dca[i].totalAmount)
        var currentPrice = amount * dca[i].currentPrice
        var feeValue = currentPrice * fee

        dcaValue = parseFloat(dcaValue) + parseFloat(currentPrice - feeValue)
    }
    dcaAmount = parseFloat(dcaAmount)
    dcaCost = parseFloat(dcaCost).toFixed(6)
    dcaValue = parseFloat(dcaValue).toFixed(6)

    dcaCostCurrency = (dcaCost * marketPrice) * currencyPrice
    dcaValueCurrency = (dcaValue * marketPrice) * currencyPrice

    document.getElementById("chDCAAmount").innerHTML = dcaAmount
    document.getElementById("chDCACost").innerHTML = dcaCost + " <small>(" + currency + " " + (dcaCostCurrency).toFixed(2) + ")</small>"
    document.getElementById("chDCAValue").innerHTML = dcaValue + " <small>(" + currency + " " + (dcaValueCurrency).toFixed(2) + ")</small>"

    dcaProfitPercentage = ((dcaValue - dcaCost) / dcaCost) * 100 || 0
    dcaProfitCurrency = ((dcaValue - dcaCost) * marketPrice) * currencyPrice
    dcaElement = document.getElementById("chDCAProfit")
    dcaElement.innerHTML = (dcaProfitPercentage).toFixed(2) + "% <small>(" + currency + " " + (dcaProfitCurrency).toFixed(2) + ")</small>"
    dcaElement.classList.remove("gray")
    dcaElement.classList.add(dcaProfitPercentage > 0 ? "text-success" : dcaProfitPercentage < 0 ? "text-danger" : "gray")

    dustAmount = dust.length
    dustCost = 0
    dustValue = 0
    dustProfit = 0
    for (i = 0; i < dustAmount; i++) {
        dustCost = parseFloat(dustCost) + parseFloat(dust[i].totalCost)
        var fee = dust[i].fee / 100
        var amount = (dust[i].totalAmount)
        var currentPrice = amount * dust[i].currentPrice
        var feeValue = currentPrice * fee

        dustValue = parseFloat(dustValue) + parseFloat(currentPrice - feeValue)
    }
    dustAmount = parseFloat(dustAmount)
    dustCost = parseFloat(dustCost).toFixed(6)
    dustValue = parseFloat(dustValue).toFixed(6)

    dustCostCurrency = (dustCost * marketPrice) * currencyPrice
    dustValueCurrency = (dustValue * marketPrice) * currencyPrice

    document.getElementById("chDustAmount").innerHTML = dustAmount
    document.getElementById("chDustCost").innerHTML = dustCost + " <small>(" + currency + " " + (dustCostCurrency).toFixed(2) + ")</small>"
    document.getElementById("chDustValue").innerHTML = dustValue + " <small>(" + currency + " " + (dustValueCurrency).toFixed(2) + ")</small>"

    dustProfitPercentage = ((dustValue - dustCost) / dustCost) * 100 || 0
    dustProfitCurrency = ((dustValue - dustCost) * marketPrice) * currencyPrice
    dustElement = document.getElementById("chDustProfit")
    dustElement.innerHTML = (dustProfitPercentage).toFixed(2) + "% <small>(" + currency + " " + (dustProfitCurrency).toFixed(2) + ")</small>"
    dustElement.classList.remove("gray")
    dustElement.classList.add(dustProfitPercentage > 0 ? "text-success" : dustProfitPercentage < 0 ? "text-danger" : "gray")

}

function profitFields(stats, marketPrice, tcv, market, currency, currencyPrice) {
    document.getElementsByClassName("marketField").innerHTML = market
    document.getElementsByClassName("currencyField").innerHTML = currency

    // TCV
    document.getElementById("totalCurrentValueField").innerHTML = tcv.toFixed(4)
    document.getElementById("totalCurrentValueUSDField").innerHTML = ((tcv * marketPrice) * currencyPrice).toFixed(2)

    // Profit Today
    document.getElementById("profifTodayPerc").innerHTML = stats.totalProfitPercToday.toFixed(2) + "%"
    var ProfitTodayUSDElement = document.getElementById("ProfitTodayUSDField")
    ProfitTodayUSDElement.innerHTML = ((stats.totalProfitToday * marketPrice) * currencyPrice).toFixed(2)
    ProfitTodayUSDElement.parentElement.classList.remove("orange")
    ProfitTodayUSDElement.parentElement.classList.add(stats.totalProfitPercToday > 0 ? "text-success" : stats.totalProfitPercToday < 0 ? "text-danger" : "orange")

    // Profit Yesterday
    document.getElementById("profifYesterdayPerc").innerHTML = stats.totalProfitPercYesterday.toFixed(2) + "%"
    var ProfitYesterdayUSDElement = document.getElementById("ProfitYesterdayUSDField")
    ProfitYesterdayUSDElement.innerHTML = ((stats.totalProfitYesterday * marketPrice) * currencyPrice).toFixed(2)
    ProfitYesterdayUSDElement.parentElement.classList.remove("orange")
    ProfitYesterdayUSDElement.parentElement.classList.add(stats.totalProfitPercYesterday > 0 ? "text-success" : stats.totalProfitPercYesterday < 0 ? "text-danger" : "orange")

    // Profit Week
    document.getElementById("profifWeekPerc").innerHTML = stats.totalProfitPercWeek.toFixed(2) + "%"
    var ProfitWeekUSDElement = document.getElementById("ProfitWeekUSDField")
    ProfitWeekUSDElement.innerHTML = ((stats.totalProfitWeek * marketPrice) * currencyPrice).toFixed(2)
    ProfitWeekUSDElement.parentElement.classList.remove("orange")
    ProfitWeekUSDElement.parentElement.classList.add(stats.totalProfitPercWeek > 0 ? "text-success" : stats.totalProfitPercWeek < 0 ? "text-danger" : "orange")
}

function currentValueDistChart(balance, pair, dca, pending) {
    new Chart(document.getElementById("cvdChart"),
        {
            "type":"doughnut",
            "data": {
                "labels": [
                    i18next.t('proToolsPage.CurrentValueDistributionChart.Balance'),
                    i18next.t('proToolsPage.CurrentValueDistributionChart.Pairs'),
                    i18next.t('proToolsPage.CurrentValueDistributionChart.DCA'),
                    i18next.t('proToolsPage.CurrentValueDistributionChart.Pending')
                ],
                "datasets":[
                    {
                        "label":"Value Distribution",
                        "data":[
                            balance.toFixed(8),
                            pair.toFixed(8),
                            dca.toFixed(8),
                            pending.toFixed(8)
                        ],
                        "backgroundColor":[
                            "#FF9900",
                            "#3bafda",
                            "#00b19d",
                            "#ef5350"
                        ]
                    }
                ]
            },
            "options": {
                "maintainAspectRatio": false,
                "plugins": {
                    "labels": [
                        {
                            "render": 'percentage',
                            "fontColor": "#000",
                            "precision": 0,
                            "showZero": false,
                        },
                        {
                            "render": "label",
                            "position": "outside",
                            "arc": true,
                            "fontColor": '#fff',
                        }
                    ]

                },
                "legend": {
                    "display": false,
                },
                "animation": false
            }
        });
}

function formatChartDate(dt){
	dt = dt.split("-")
	var newDate = new Date(dt[2]+"-"+dt[1]+"-"+dt[0])
	newDate = newDate.getMonth()+1+'/'+newDate.getDate();
	return newDate
}


function profitablePairsChart(labels,data){
    var mppContext = document.getElementById("mppChart").getContext("2d");
    var mppData = {
        labels: labels,
        datasets: [
            {
                label: [],
                data: data,
                backgroundColor: "#3bafda",
                borderWidth: 0
            }]
    };

    var mppChart = new Chart(mppContext, {
          type: 'horizontalBar',
          data: mppData,
          options: {
              legend: {
                display: false,
              },
              animation: false,
              scales: {
                  xAxes: [{
                      ticks: {
                        fontColor: "rgba(152,166,173,0.5)",
                      },
                      gridLines: {
            					  display: true,
            					  color: "rgba(152,166,173,0.1)",
                        drawBorder: false,
                      }
                  }],
                  yAxes: [{
                      stacked: true,
                      ticks: {
                        fontColor: "rgba(152,166,173,0.5)",
                      },
                      gridLines: {
            					  display: false,
            					  color: "rgba(152,166,173,0.1)",
                        drawBorder: false,
                      }
                  }]
              }
          }
      });

}


function dailyCummulativeChart(stats) {
  if (!stats) {
    return;
  }
    stats.reverse()
    var labels = [], profits = [], cProfits = []

    for (i = 0; i < stats.length; i++) {
        labels.push(formatChartDate(stats[i].date))

        profit = (stats[i].totalProfitCurrency).toFixed(5)
        profits.push(parseFloat(profit))

        if(i == 0) {
            cProfits.push(parseFloat(profit))
        } else {
            var calcs = (parseFloat(profit) + parseFloat(cProfits[i - 1])).toFixed(5)

            cProfits.push(calcs)
        }


    }

    new Chart(document.getElementById("dcChart"),
    {
        "type":"line",
        "data": {
            "labels": labels,
            "datasets":[
                {
                    "label": i18next.t('proToolsPage.DailyProfits'),
                    "data": profits,
                    "fill":false,
                    "borderColor":"rgb(75, 192, 192)",
                    "pointRadius": 0,
                    "lineTension":0.4
                },
                {
                    "label":i18next.t('proToolsPage.CumulativeProfits'),
                    "data": cProfits,
                    "fill":false,
                    "borderColor":"#ef5350",
                    "lineTension":0.4,
                    "pointRadius": 0,
                }
            ]
        },
        "options":{
            "responsive": true,
            "maintainAspectRatio": false,
            "legend": {
                "display": false,
            },
            "animation": false,
            "tooltips": {
                "mode": "nearest",
                "intersect": false,
            },
            "hover": {
                "mode": "nearest",
                "intersect": true,
            },

            "scales": {
                "yAxes": [{
                    "ticks": {
                        "fontColor":"rgba(152,166,173,0.5)",
                    },
                "gridLines": {
                  "display": true,
                  "color": "rgba(152,166,173,0.1)",
                  "zeroLineColor": "rgba(152,166,173,0.1)",
                }
            }],
				      "xAxes": [{
                "ticks": {
                  "fontColor":"rgba(152,166,173,0.5)",
                  "maxTicksLimit": 10
                },
					"gridLines": {
					  "display": true,
					  "color": "rgba(152,166,173,0.1)",
					}
                }]
            }
        }
    });
}

function getSparklineData(currency,market){
    var sparklineData = []
    if(market == 'USD') {
        sparklineData.push(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)
        $('#currencyPrice').html('1.00');
        $('#currencyPriceChange').html('+0.00%').attr('class','delta label label-success');
        drawSparkline(sparklineData)
    } else {
        jQuery.get('https://min-api.cryptocompare.com/data/histominute?limit=60&fsym='+market+'&tsym='+currency, function (response) {
            var data = response.Data
            for (i = 0; data && i < data.length; i++) {
                sparklineData.push(data[i].close)
            }
            if(sparklineData[sparklineData.length-1] > sparklineData[0]){
                priceChange = ((sparklineData[sparklineData.length-1]-sparklineData[0])/sparklineData[0])*100
                $('#currencyPriceChange').html('+'+priceChange.toFixed(2)+"%").attr('class','delta label label-success');
            }else{
                priceChange = ((sparklineData[0]-sparklineData[sparklineData.length-1])/sparklineData[0])*100
                $('#currencyPriceChange').html('-'+priceChange.toFixed(2)+"%").attr('class','delta label label-danger');
            }
            var displayCurrency = sparklineData.length ? sparklineData[sparklineData.length - 1].toFixed(2) : 0.00;
            $('#currencyPrice').html(displayCurrency);
            drawSparkline(sparklineData)
        })
    }
}

function drawSparkline(data){
	$("#sparkline").sparkline(data, {
	type: 'line',
	width: '100%',
	height: '47px',
	lineColor: '#fff',
	fillColor: 'rgba(64,76,88,0.9)',
	lineWidth: 3,
	spotColor: '#f08000',
	minSpotColor: '#f08000',
	maxSpotColor: '#f08000',
	highlightSpotColor: undefined,
	highlightLineColor: undefined,
	spotRadius: 1.5 });
}


(function () {

    /**
     * Decimal adjustment of a number.
     *
     * @param   {String}    type    The type of adjustment.
     * @param   {Number}    value   The number.
     * @param   {Integer}   exp     The exponent (the 10 logarithm of the adjustment base).
     * @returns {Number}            The adjusted value.
     */
    function decimalAdjust(type, value, exp) {
        // If the exp is undefined or zero...
        if (typeof exp === 'undefined' || +exp === 0) {
            return Math[type](value);
        }
        value = +value;
        exp = +exp;
        // If the value is not a number or the exp is not an integer...
        if (isNaN(value) || !(typeof exp === 'number' && exp % 1 === 0)) {
            return NaN;
        }
        // Shift
        value = value.toString().split('e');
        value = Math[type](+(value[0] + 'e' + (value[1] ? (+value[1] - exp) : -exp)));
        // Shift back
        value = value.toString().split('e');
        return +(value[0] + 'e' + (value[1] ? (+value[1] + exp) : exp));
    }

    // Decimal round
    if (!Math.round10) {
        Math.round10 = function (value, exp) {
            return decimalAdjust('round', value, exp);
        };
    }
    // Decimal floor
    if (!Math.floor10) {
        Math.floor10 = function (value, exp) {
            return decimalAdjust('floor', value, exp);
        };
    }
    // Decimal ceil
    if (!Math.ceil10) {
        Math.ceil10 = function (value, exp) {
            return decimalAdjust('ceil', value, exp);
        };
    }
})();

function isFloat(n) {
    return Number(n) === n && n % 1 !== 0;
}

$.fn.autocolor = function (isSawTableCell = false) {
    return this.each(function (index) {
        var element = $(this);

        if (isSawTableCell) {
            element = $(".tablesaw-cell-content", this);
        }

        var text = $(element).text().replace(/[^\d.,-]/g, '');
        var floatVal = parseFloat(text);

        element.removeClass('text-danger');
        element.removeClass('text-success');
        if (floatVal > 0) {
            element.addClass('text-success');
        } else {
            element.addClass('text-danger');
        }
    });
};

// To render datatable for Pro Tools tab.
function renderDailyProfitsTable(logData) {
  // Defining columns which are to be used in daily profits datatable.
  var column = [
    {
      title: i18next.t('dailyProfitTable.date.colName'),
      tooltip: 'dailyProfitTable.date.colTitle',
      data: DataTableHelper.renderDateForDailyProfit(),
      className: 'date blue'
    },
    {
      title: i18next.t('dailyProfitTable.sells.colName'),
      tooltip: 'dailyProfitTable.sells.colTitle',
      data: 'totalSales'
    },
    {
      title: i18next.t('dailyProfitTable.boughtCost.colName'),
      tooltip: 'dailyProfitTable.boughtCost.colTitle',
      data: 'totalBoughtCost',
      render: DataTableHelper.addPrecision,
      className: 'yellow'
    },
    {
      title: i18next.t('dailyProfitTable.soldValue.colName'),
      tooltip: 'dailyProfitTable.soldValue.colTitle',
      data: 'totalSoldValue',
      render: DataTableHelper.addPrecision,
      className: 'orange'
    },
    {
      title: i18next.t('dailyProfitTable.profits.colName'),
      tooltip: 'dailyProfitTable.profits.colTitle',
      data: 'totalProfitCurrency',
      render: DataTableHelper.handleProfitForDailySection,
      className: 'text-center'
    },
    {
      title: i18next.t('dailyProfitTable.profitPerc.colName'),
      tooltip: 'dailyProfitTable.profitPerc.colTitle',
      data: 'avgProfit',
      render: DataTableHelper.renderProfitPercentage,
      className: 'text-center'
    },
  ];
  if (isDataTable) {
    // If daily profits datatable is defined then rebinding data, rather than redefining datatable.
    DataTableHelper.reBindData(dailyProfitTable, logData)
  } else {
    // If daily profits datatable is not defined then defining it or is destroyed.
    dailyProfitTable = $('#dtDailyProfit').DataTable(
      DataTableHelper.getDataTableStructure({
        columns: column,
        stateSave: true,
        paging: true,
        data: logData,
        language: {
          // changes the total records text shown at the bottom.
          info: i18next.t('dtMsgs.totalRecords') + ': _TOTAL_',
          infoEmpty: i18next.t('dtMsgs.totalRecords') + ': 0',
          emptyTable: i18next.t('dtMsgs.noRecords'),
          infoFiltered: i18next.t('dtMsgs.filtered')
        },
        ordering: false,
        searching: false,
        dom: 'ltip',
      })
    );
    isDataTable = true;
  }
}

function addEventListenerForProTools() {
  // To destroy daily profit datatable when we move out of pro-tools tab.
  $(document).on('evt.destroy-pro-tools-datatable', function () {
    if (isDataTable) {
      dailyProfitTable.destroy();
      isDataTable = false;
    }
  });

  // To recreate table when we switch to pro-tools tab.
  $(document).on('evt.render-daily-profit-datatable', function () {
    if (serverData) {
      renderDailyProfitsTable(serverData.proStats.dailyStats);
    } else {
      renderDailyProfitsTable();
    }
  });

  $(document).on('evt.rebind-click-event-in-protool', function () {
    $('.dt-current-holdings tr > td:nth-child(1) > a').unbind();
    $('.dt-current-holdings tr > td:nth-child(1) > a').click(function (e) {
      e.preventDefault();
      var href = $(this).attr('href');
      page(href);
      // To remove active class from navbar element.
      $('a[href="pro-tools"]').parent().removeClass('active');

      // To add active class for active tab.
      $('a[href="' + href + '"]').parent().addClass('active');
    });
  });
}
