var DataTableHelper = (function() {
  // A cache for data that has a global impact on rendered columns
  // but changes whenever an AJAX call is made. For example - Marketing Site.
  var dtCache = {};
  const monthNames = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
  var displayTypesArr = ['filter', 'display', 'export', 'sort'];
  var sortOperation = 'sort';
  var MAX_TAM_PRECISION = 4;
  var MAX_BUY_BOUGHT_AMNT_PRECISION = 8;
  var getOnlyCurrentValueForSalesLog = getCurrentValueForSalesLog(true);
  var handleOnlyTotalCstFrSalesLog = handleTotalCstFrSalesLog(true);
  var getOnlyCurrCurrencyValForSalesLog = getCurrentValueForSalesLog(
    false,
    true
  );
  var handleOnlyTotalCstCurrencyValFrSalesLog = handleTotalCstFrSalesLog(
    false,
    true
  );
  var handleOnlyAvgPrice = handleAvgPriceBoughtTimes(true);
  var scale = chroma.scale(['#FF9900', '#3bafda']);
  var POSITIVE_TRUE_VALUE = 1;
  var POSITIVE_TRUE_TRAILING_VALUE = 1.1;
  var MAX_BOUGHT_TIMES = 200;

  var stratValMaxPrecision = 2;
  var FIXED_COLUMN_MAX_RESOLUTION = '767';
  var isLowerResolution = false;
  var crntPaginationNumber = 0;
  var NormalizerService = Services.Normalizer();

  // We have to show scrollbar and make first column fixed in lower resolutions.
  if ($(window).innerWidth() <= FIXED_COLUMN_MAX_RESOLUTION) {
    isLowerResolution = true;
  }

  function isFixedColumnsResolution() {
    return isLowerResolution;
  }

  /**
   * Date column should be split in excel
   * In excel we should show date, time. difference in three different columns.
   * @param {*} columns
   * @param {*} index
   * @param {*} property
   * @param {*} parentProperty
   */
  function getExportDateColumns(columns, index, property, parentProperty) {
    var exportDateColumns = [
      {
        title: i18next.t('dataTableSection.date.colName'),
        data: dateHandler(property, parentProperty, true),
        visible: false
      },
      {
        title: i18next.t('dataTableSection.time.colName'),
        data: dateHandler(property, parentProperty, false, true),
        visible: false
      },
      {
        title: i18next.t('dataTableSection.timeSince.colName'),
        data: dateHandler(property, parentProperty, false, false, true),
        visible: false
      }
    ];

    for (var i = 0; i < exportDateColumns.length; i++) {
      columns.splice(index + i, 0, exportDateColumns[i]);
    }

    return columns;
  }

  function dateHandler(propName, parentProperty, onlyDate, onlyTime, onlyDiff) {
    // return the actual render function
    return function(row, type) {
      // To return -- when we are receiving parentProperty but the row doesnot contain parentProperty.
      if (typeof parentProperty !== 'undefined' && typeof row[parentProperty] === 'undefined') {
        return '--';
      }
      var colData = parentProperty
        ? row[parentProperty][propName]
        : row[propName];

      // If colData is not a date object then assuming it to contain timestamp and creating date object using it.
      // Javascript uses milliseconds internally, while normal UNIX timestamps are usually in seconds so multiplying 1000 with it.
      const dateObj = typeof colData === 'object' ? colData : new Date(colData * 1000);

      // TODO: Have to add '+00:00' to date to specify it is in UTC timezone.
      var dt = DateTimeHelper.getDateObj(dateObj, dtCache.timeZoneOffset, true);
      if (!dt) {
        return '--';
      }
      if (type === 'set') {
        return dt.getTime();
      } else if (type === 'display' || type === 'export' || type === 'export') {
        var date = DateTimeHelper.formatDate(dt);
        var dateAndTimeArr = date.split(' ');
        // Add time period(AM or PM) if exists.
        var timePeriod = dateAndTimeArr[2] ? ' ' + dateAndTimeArr[2] : '';
        var diffDays = getDaysDifferenceFromToday(dt);
        var diffDaysString = diffDays + 'D';
        if (onlyDate) {
          return dateAndTimeArr[0];
        }
        if (onlyTime) {
          return dateAndTimeArr[1] + timePeriod;
        }
        if (onlyDiff) {
          return diffDaysString;
        }
        var dateAndDaysDiff =
          '<span>' +
          dateAndTimeArr[0] +
          '</span><br><span class="dt-time">' +
          ' ' +
          dateAndTimeArr[1] +
          timePeriod +
          ' (' +
          diffDaysString +
          ')</span>';
        dateAndDaysDiff = replaceBreakTag(dateAndDaysDiff, type);
        return dateAndDaysDiff;
      } else if (type === 'filter') {
        return DateTimeHelper.formatDate(dt);
      }
      // 'sort', 'type' and undefined all just use the integer
      return dt.getTime();
    };
  }

  /**
   * This will return current date and time which will be added to exported file names.
   */
  function getCurrentDateForFileName() {
    return DateTimeHelper.getCurrentDateForFileName();
  }

  function getDaysDifferenceFromToday(date) {
    var currentDate = DateTimeHelper.getUTCDateWithOffset(
      dtCache.timeZoneOffset
    );
    var timeDiff = date.getTime() - currentDate.getTime();
    var diffDaysTmp = timeDiff / (1000 * 3600 * 24);
    var diffDays =
      diffDaysTmp < 0 ? Math.ceil(diffDaysTmp) : Math.floor(diffDaysTmp);
    return diffDays;
  }

  function handleProfit(data, type) {
    var profit = handleMoney(data, type);
    var profitClass = +profit < PBConstants.PROFIT_GREEN ? 'danger' : 'success';
    var profitString =
      '<span class="badge badge-' + profitClass + '">' + profit + '</span>';
    return profitString;
  }

  function handleProfitText(td, cellData) {
    // To add custom formatting to the cell based on the value.
    // https://datatables.net/reference/option/columns.createdCell
    if (cellData < PBConstants.PROFIT_GREEN) {
      $(td)
        .addClass('loss-classtext')
        .removeClass('profit-classtext');
    } else {
      $(td)
        .addClass('profit-classtext')
        .removeClass('loss-classtext');
    }
  }

  function renderMarketCol(data, type, row) {
    if (type === 'display' || type === 'export') {
      if (
        !dtCache.marketingSite.indexOf('%c') ||
        !dtCache.marketingSite.indexOf('%b')
      ) {
        return data;
      }
      let normalizedMarketSite = dtCache.marketingSite.replace(
        '%c',
        row.currency
      );
      normalizedMarketSite = normalizedMarketSite.replace('%b', row.base);
      return (
        '<a href="' +
        normalizedMarketSite +
        '" target="_blank" data-order="'+row.orderNumber+'">' +
        data +
        '</a>'
      );
    }
    return data;
  }

  function handleMoney(data, type) {
    if (type === 'display' || type === 'export') {
      // Determining the return value based on the data's type as toFixed() only works with number type.
      var returnValue = isNaN(parseInt(data, 10))
        ? ''
        : parseFloat(parseFloat(data).toFixed(2)).toFixed(2);
      return returnValue;
    }
    return data;
  }

  function setIndex(dtApi) {
    dtApi
      .on('order.dt', function() {
        dtApi
          .column(0, { order: 'applied' })
          .nodes()
          .each(function(cell, i) {
            cell.innerHTML = i + 1;
          });
      })
      .draw();
  }

  function addPrecision(data, type) {
    return checkAndAddPrecision(data, type);
  }

  function getExcelColumns(totalColums, excludeInExcel, excludeOnCondition) {
    // Generate excel columns
    var columnsArrTmp = [];
    totalColums = totalColums - 1;
    for (var i = 0; i <= totalColums; i++) {
      // In excel we shouldn't show combined values
      if (excludeInExcel.indexOf(i) === -1) {
        columnsArrTmp.push(i);
      }
    }
    var columnsArr = columnsArrTmp;
    // If market is USDT and currency is USD then hide values in current currency.
    if (
      excludeOnCondition &&
      excludeOnCondition.length > 0 &&
      dtCache.market === PBConstants.HIDE_PROFIT_MARKET_AND_ESTIMATED_USD &&
      dtCache.currency === PBConstants.BASE_CURRENCY
    ) {
      for (var j = 0; j < excludeOnCondition.length; j++) {
        columnsArr.splice(columnsArr.indexOf(excludeOnCondition[j]), 1);
      }
    }
    return columnsArr;
  }

  function getDataTableStructure(dtCustom) {
    var fixedColumns =
      isLowerResolution && dtCustom.fixedColumnsLength
        ? dtCustom.fixedColumnsLength
        : 0;
    delete dtCustom.fixedColumnsLength;
    return jQuery.extend(
      true,
      {
        paging: false,
        dom: 'frtipB',
        scrollX: isLowerResolution,
        responsive: !isLowerResolution,
        stateSave: true,
        language: {
          // changes the total records text shown at the bottom.
          info: i18next.t('dtMsgs.totalRecords') + ': _TOTAL_',
          infoEmpty: i18next.t('dtMsgs.totalRecords') + ': 0',
          emptyTable: i18next.t('dtMsgs.noRecords'),
          infoFiltered: i18next.t('dtMsgs.filtered')
        },
        searching: false,
        initComplete: function() {
          jQuery(document).trigger('evt.add-market-value');
        },
        fixedColumns: {
          leftColumns: fixedColumns
        },
        scrollCollapse: true
      },
      dtCustom
    );
  }

  function reBindData(table, data) {
    // if data is undefined, set it to an empty array
    // so that datatable doesn't crash.
    if (!data) {
      data = [];
    }
    table.clear().rows.add(data);
    table.draw(false);
  }

  function getCurrentValue(onlyCurrentValue, onlyConvertedCurrentValue) {
    return function(data, type) {
      var currentValue;

      /* As only Pending, DCA and PAIRS Log contains currentValue property but Dust Log doesn't and uses this function, we are
      checking if data object contains currentValue property before using it. */
      if (data.hasOwnProperty('currentValue')) {
        currentValue = data.currentValue;
      } else {
        currentValue = data.totalAmount * data.currentPrice;
      }
      var nrmlsdCurrentValue = getCurrentValueBsdOnfee(data, currentValue);
      return getValueBasedOnParams(
        nrmlsdCurrentValue,
        type,
        onlyCurrentValue,
        onlyConvertedCurrentValue
      );
    };
  }

  function getCurrentValueBsdOnfee(data, currentValue) {
    var fee = data.fee;
    currentValue = currentValue * (1 - fee / 100);
    return currentValue;
  }

  function checkAndAddPrecision(data, type, precision) {
    if (
      type === 'display' ||
      type === 'export' ||
      type === 'sales' ||
      type === 'filter'
    ) {
      // If precision is not a number then consider default precision.
      precision = parseInt(precision);
      var precisionTmp = isNaN(precision)
        ? PBConstants.DEFAULT_PRECISION
        : precision;
      // Determining the return value based on the data's type as toFixed() only works with number type.
      var returnValue = isNaN(parseInt(data, 10))
        ? ''
        : parseFloat(parseFloat(data).toFixed(precisionTmp)).toFixed(precisionTmp);
      return returnValue;
    }
    return data;
  }

  function renderPercentageChange(data, type, row) {
    var percChange = row.percChange * 100;
    // Determining the return value based on the data's type as toFixed() only works with number type.
    var returnValue = isNaN(parseInt(percChange, 10))
      ? ''
      : parseFloat(parseFloat(percChange).toFixed(2)).toFixed(2);
    if (type === 'export') {
      return returnValue + ' %';
    }
    return returnValue;
  }

  function handleTotalAmnt(data, type) {
    return checkMaxDecimalsAndAddPrecision(
      data.totalAmount,
      type,
      MAX_TAM_PRECISION
    );
  }

  function handleBuyLogBoughtAmnt(data, type) {
    var totalAmnt = data.totalAmount ? data.totalAmount : '';
    return checkMaxDecimalsAndAddPrecision(
      totalAmnt,
      type,
      MAX_BUY_BOUGHT_AMNT_PRECISION
    );
  }

  function removeTrailingZeros(data, type) {
    if (displayTypesArr.indexOf(type) !== -1) {
      return normalizeValueWithCommas(parseFloat(data), type, '', '', true);
    }
    return data;
  }

  function handleTotalCst(onlyCurrentValue, onlyConvertedCurrentValue) {
    return function(data, type) {
      var totalCst = data.totalCost ? data.totalCost : '';
      // If total cost is not set then show empty column.
      if (typeof totalCst === 'undefined') {
        return '';
      }

      return getValueBasedOnParams(
        totalCst,
        type,
        onlyCurrentValue,
        onlyConvertedCurrentValue
      );
    };
  }

  function getValueBasedOnParams(
    value,
    type,
    onlyCurrentValue,
    onlyConvertedCurrentValue
  ) {
    var onlyValue = normalizeValueWithCommas(
      value,
      type,
      '',
      '',
      '',
      dtCache.currentMarketPrecision
    );
    if (onlyCurrentValue) {
      return onlyValue;
    }
    if (onlyConvertedCurrentValue) {
      return convertToCurrentCurrency(value, type);
    }
    return onlyValue + convertToCurrentCurrency(value, type);
  }

  function handleTotalCstFrSalesLog(
    onlyCurrentValue,
    onlyConvertedCurrentValue
  ) {
    return function(data, type) {
      var totalCost = data.totalCost;
      return getValueBasedOnParams(
        totalCost,
        type,
        onlyCurrentValue,
        onlyConvertedCurrentValue
      );
    };
  }

  function getCurrentValueForSalesLog(
    onlyCurrentValue,
    onlyConvertedCurrentValue
  ) {
    return function(data, type) {
      var currentValue = data.soldValue;
      currentValue = getCurrentValueBsdOnfee(data, currentValue);
      return getValueBasedOnParams(
        currentValue,
        type,
        onlyCurrentValue,
        onlyConvertedCurrentValue
      );
    };
  }

  /**
   * It will return "BBLow" or "BBHigh" that has a value.
   * It will return zero if both have zero value.
   */
  function handleCurrentBB(data, type) {
    return checkAndAddPrecision(data.triggerValue, type);
  }

  function handleAvgPrice(data, type) {
    return currentMarketPrecision(
      data.avgPrice,
      type,
      '',
      '',
      '',
      dtCache.currentMarketPrecision
    );
  }

  function handleAvgPriceDcaLog(onlyBid, onlyAvg) {
    return function(data, type) {
      if (type === 'display' || type === 'export') {
        var currentPrice = normalizeValueWithCommas(
          data.currentPrice,
          type,
          '',
          '',
          '',
          dtCache.currentMarketPrecision
        );
        var avgPrice = normalizeValueWithCommas(
          data.avgPrice,
          type,
          '',
          '',
          '',
          dtCache.currentMarketPrecision
        );
        if (onlyBid) {
          return currentPrice;
        }
        if (onlyAvg) {
          return (
            avgPrice +
            '<span class="dca-bought-times-container">(' +
            data.boughtTimes +
            ')</span></span>'
          );
        }
        // To get background color based on the boughtTimes value that we get from API.
        var avgPriceContent = avgPrice + renderBoughtTimesBasedOnValue(data.boughtTimes);
        return (
          '<span class="dca-avg-current-price">' +
          currentPrice +
          getBoughtTimesHTML(data.boughtTimes, false) +
          '</br>' +
          avgPriceContent +
          '</span>'
        );
      } else if (type === 'sort') {
        return data.boughtTimes;
      }
      return data.avgPrice;
    };
  }

  /**
   * It will get current pagination number from localstorage and set to variable.
   *
   */
  function getCurrentPaginationNumber(dataTableId) {
    var currentPath = window.location.pathname;
    crntPaginationNumber = 0;
    var tableLSData = JSON.parse(
      localStorage.getItem('DataTables_' + dataTableId + '_' + currentPath)
    );
    if (tableLSData) {
      crntPaginationNumber =
        tableLSData.start && tableLSData.length
          ? tableLSData.start / tableLSData.length
          : 0;
    }
  }

  /**
   * Pagination was not being saved, everytime on page refresh it was showing from zero
   * It was due to on page load we are intializing table with zero columns and adding
   * data after API call, because of initializing datatable with zero columns pagination was becoming
   * zero. To fix that we are storing pagination number before intializing and setting that page number
   * after data rebinding.
   */
  function setDTPagination(table) {
    crntPaginationNumber ? table.page(crntPaginationNumber).draw(false) : '';
  }

  function renderDCAEntryAndEntryLimit(data, type) {
    var entryLimitString = '';
    // TODO: Check why we are receiving type as a type.
    if (type === sortOperation || type === 'type') {
      return data[0].triggerValue;
    }
    for (var i = 0; i < data.length; i++) {
      var currentStrat = data[i];
      entryLimitString += currentStrat.anyValueExists
        ? '<span class="strat-trigger-value">' +
          currentStrat.triggerValue +
          '</span>'
        : '';
      entryLimitString += '<br>';
    }
    entryLimitString = replaceBreakTag(entryLimitString, type);
    return entryLimitString;
  }

  function getProfitBTCForSalesLog(row, type) {
    var profitValueTmp = row.profitCurrency;
    var profitValue = normalizeValueWithCommas(profitValueTmp, type);
    if (type === 'display' || type === 'export' || type === 'filter') {
      var profitClass = getClassBasedOnValue(profitValueTmp);
      var profitString =
        '<span class="' + profitClass + '">' + profitValue + '</span>';
      return profitString;
    }
    return profitValue;
  }

  function getCurrentTrendprofit(row, type) {
    // If market is USDT then don't show the market profit value.
    if (
      dtCache.market === PBConstants.HIDE_PROFIT_MARKET_AND_ESTIMATED_USD &&
      dtCache.currency === PBConstants.BASE_CURRENCY
    ) {
      return '';
    }
    var profitValue = getProfitBTCForSalesLog(row, '');
    var currentTrendProfitTmp = dtCache.marketPrice * profitValue * dtCache.currencyValue;
    var currentTrendProfit = normalizeValueWithCommas(
      currentTrendProfitTmp,
      type,
      '',
      '',
      '',
      2
    );
    var currentTrendProfitClass = getClassBasedOnValue(currentTrendProfitTmp);
    return (
      '<span class="sales-market-profit ' +
      currentTrendProfitClass +
      '"> ' +
      dtCache.currencySymbol + ' ' +
      currentTrendProfit +
      '<span>'
    );
  }

  function getClassBasedOnValue(value) {
    var valueClass =
      +value < PBConstants.PROFIT_GREEN ? 'loss-classtext' : 'profit-classtext';
    return valueClass;
  }

  function addButtons(dataTable, buttons, tableId) {
    new $.fn.dataTable.Buttons(dataTable, {
      buttons: buttons
    });
    $('#' + tableId + '_wrapper')
      .find('.dt-buttons')
      .remove();
    dataTable
      .buttons(0, null)
      .container()
      .last()
      .appendTo(dataTable.table().container());
  }

  function convertToCurrentCurrency(value, type) {
    // Converted value should be added only for type display and filter.
    if (displayTypesArr.indexOf(type) === -1) {
      return 0;
    }
    // If market is USDT and currency is USD then converted value should not be shown.
    if (
      dtCache.market === PBConstants.HIDE_PROFIT_MARKET_AND_ESTIMATED_USD &&
      dtCache.currency === PBConstants.BASE_CURRENCY
    ) {
      return '';
    }
    value = normalizeValueWithCommas(
      value * dtCache.marketPrice * dtCache.currencyValue,
      type,
      '',
      '',
      '',
      2
    );
    // Returns the value directly if the type of operation is sorting.
    if (type === sortOperation) {
      return value;
    }
    // Returns the value with prefixed currency symbol if the type of operation is not sorting.
    value = dtCache.currencySymbol + ' ' + value;
    return value;
  }

  function renderStrategy(data, type) {
    var strategy = '';
    var positiveCls = '';

    // Loop over strategies array.
    for (var i = 0; i < data.length; i++) {
      var currentStrategy = data[i];
      data[i] = checkAnyValueExists(data[i]);
      strategy += currentStrategy.name;
      if (currentStrategy.positive && currentStrategy.positive !== 'false') {
        positiveCls = PBConstants.POSITIVE_CLASS;
        strategy +=
          '<span class="' +
          positiveCls +
          '"> (' +
          currentStrategy.positive.toString() +
          ')</span><br>';
      } else {
        positiveCls = PBConstants.NEGATIVE_CLASS;
        strategy += '<br>';
      }
    }
    strategy = replaceBreakTag(strategy, type);
    return strategy;
  }

  function renderSellStrategy(data, type) {
    var sellStrategy = '';
    var positiveCls = '';

    // Loop over sell strategy array.
    for (var j = 0; j < data.length; j++) {
      var currentSellStrategy = data[j];
      sellStrategy += currentSellStrategy.name;
      if (dtCache.displayAdvancedStats && currentSellStrategy.positive && currentSellStrategy.positive !== 'false') {
        positiveCls = PBConstants.POSITIVE_CLASS;
        sellStrategy +=
          '<span class="' +
          positiveCls +
          '"> (' +
          currentSellStrategy.positive.toString() +
          ')</span><br>';
      } else {
        positiveCls = PBConstants.NEGATIVE_CLASS;
        sellStrategy += '<br>';
      }
    }
    sellStrategy = replaceBreakTag(sellStrategy, type);
    return sellStrategy;
  }

  function renderBuyStrategy(data, type) {
    var strategy = 0;
    var weightage = 0;
    var currentPositiveValue = '';
    // We have to sort on positive value for buy strategies column.
    if (type === sortOperation) {
      // Loop over strategies array.
      for (var i = 0; i < data.length; i++) {
        var currentStrategy = data[i];
        currentPositiveValue = currentStrategy.positive
          .toString()
          .toLowerCase();
        // Adding weightage based on positive value we are receiving from API.
        if (currentPositiveValue !== 'false') {
          // Implementing sorting based on the sum for each row value we get from this function.
          weightage =
            currentPositiveValue === 'true'
              ? POSITIVE_TRUE_VALUE
              : POSITIVE_TRUE_TRAILING_VALUE;
          strategy = strategy + weightage;
        }
      }
      return strategy;
    }
    // Flow of control moves to renderStrategy function if the type of operation is not sort.
    return renderStrategy(data, type);
  }

  function handleTrigger(isNotPerc, considerDecimals) {
    var precision = 2;
    // If we have to show value in percentage then we have to add precision 2 otherwise we have to add precision 8.
    if (isNotPerc) {
      precision = PBConstants.DEFAULT_PRECISION;
    }

    return function(data, type) {
      var triggerString = data.length
        ? ''
        : checkAndAddPrecision(0, type, precision);
      // Loop over strategies array.

      // In DCA logs if trigger value precision should not be greater than 2
      // if it is greater than 2 then we have to add precision 2.
      if (considerDecimals) {
        for (var i = 0; i < data.length; i++) {
          var currentStrat = data[i];
          var triggerValue = currentStrat.triggerValue;
          triggerString +=
            '<span class="strat-trigger-value">' +
            checkMaxDecimalsAndAddPrecision(
              triggerValue,
              type,
              stratValMaxPrecision
            ) +
            '</span>';
          triggerString += '<br>';
        }
      } else {
        for (var j = 0; j < data.length; j++) {
          triggerString +=
            checkAndAddPrecision(data[j].triggerValue, type, precision) +
            '<br>';
        }
      }
      triggerString = replaceBreakTag(triggerString, type);
      return triggerString;
    };
  }

  function checkMaxDecimalsAndAddPrecision(value, type, maxPrecision) {

    // If value is not set then show the empty column.
    if (typeof value === 'undefined') {
      return '';
    }

    var valueArr = value.toString().split('.');
    if (valueArr[1] && valueArr[1].length > maxPrecision) {
      value = checkAndAddPrecision(value, type, maxPrecision);
    }
    return value;
  }

  function checkAnyValueExists(strat) {
    if (
      strat.currentValue ||
      strat.triggerValue ||
      strat.entryValue ||
      strat.entryValueLimit
    ) {
      strat.anyValueExists = true;
    }
    return strat;
  }

  function renderCombinedMarketCol(data, type, row) {
    var marketValue = renderMarketCol(data, type, row);
    if (type === 'export') {
      return marketValue;
    }
    var percentageValue = renderPercentageChange(data, type, row);
    var percentageClass = getClassBasedOnValue(percentageValue);
    var marketAndPercString =
      marketValue +
      '<br>' +
      '<span class="' +
      percentageClass +
      '">' +
      percentageValue +
      ' %</span>';
    return marketAndPercString;
  }

  // In exported files also we have to show in two lines.
  // Break tag(<br>) will not work in excel.
  function replaceBreakTag(string, type) {
    if (type === 'export') {
      string = string.replace(/<br>/gi, '\r');
    }
    return string;
  }

  function handleStratEntryVal(data, type, row, set, checkAnyValueExists) {
    var entryValString = '';
    // TODO: Check why we are receiving type as a type.
    if (type === sortOperation || type === 'type') {
      return checkAndAddPrecision(data[0].entryValue, type, data[0].decimals);
    }
    // Loop over strategies array.
    for (var i = 0; i < data.length; i++) {
      var currentStrat = data[i];
      var money = checkAndAddPrecision(
        currentStrat.entryValue,
        type,
        currentStrat.decimals
      );
      // In DCA table we have to render strategy data if atleast one value is not zero.
      if (checkAnyValueExists) {
        money = currentStrat.anyValueExists ? money : '';
      }
      entryValString += money + '<br>';
    }
    entryValString = replaceBreakTag(entryValString, type);
    return entryValString;
  }

  function checkAllValAndHandleStratEntryVal(data, type) {
    return handleStratEntryVal(data, type, '', '', true);
  }

  function handlePBEntryLimit(data, type) {
    var pbEntryLimitString = '';
    // TODO: Check why we are receiving type as a type.
    if (type === sortOperation || type === 'type') {
      return checkAndAddPrecision(data[0]['entryValueLimit'], type, data[0].decimlas);
    }
    // Loop over strategies array.
    for (var i = 0; i < data.length; i++) {
      var entryValLimit = data[i].anyValueExists
        ? checkAndAddPrecision(data[i]['entryValueLimit'], type, data[i].decimals)
        : '';
      pbEntryLimitString += entryValLimit + '<br>';
    }
    pbEntryLimitString = replaceBreakTag(pbEntryLimitString, type);
    return pbEntryLimitString;
  }

  function handleStratCurrentVal(data, type, row, set, checkAnyValueExists) {
    var currentValString = '';
    // TODO: Check why we are receiving type as a type.
    if (type === sortOperation || type === 'type') {
      return checkAndAddPrecision(data[0].currentValue, type, data[0].decimals);
    }
    // Loop over strategies array.
    for (var i = 0; i < data.length; i++) {
      var currentVal = checkAndAddPrecision(
        data[i]['currentValue'],
        type,
        data[i].decimals
      );
      // In DCA table we have to render strategy data if atleast one value is not zero.
      if (checkAnyValueExists) {
        currentVal = data[i].anyValueExists ? currentVal : '';
      }
      currentValString += currentVal + '<br>';
    }
    currentValString = replaceBreakTag(currentValString, type);
    return currentValString;
  }

  function renderVolume(data) {
    return Math.round(data);
  }

  function checkAllValAndHandleStratCurVal(data, type) {
    return handleStratCurrentVal(data, type, '', '', true);
  }

  function handleAvgPriceBoughtTimes(onlyBoughtPrice, onlySoldPrice) {
    return function(data, type) {
      // Returns the first value in the column's row if the type of opertaion is sorting.
      if (type === sortOperation) {
        return data.avgPrice;
      }
      if (displayTypesArr.indexOf(type) !== -1) {
        var avgPriceBroughtTimes = normalizeValueWithCommas(
          data.avgPrice,
          type,
          '',
          '',
          '',
          dtCache.currentMarketPrecision
        );
        var currentPrice = normalizeValueWithCommas(
          data.currentPrice,
          type,
          '',
          '',
          '',
          dtCache.currentMarketPrecision
        );
        var avgPriceBroughtTimesCount = '';
        // Rendering data when user exports excel.
        if (type === 'export') {
          if (onlyBoughtPrice) {
            avgPriceBroughtTimesCount = '(' + data.boughtTimes + ')';
            return avgPriceBroughtTimes + avgPriceBroughtTimesCount;
          }
          if (onlySoldPrice) {
            return currentPrice;
          }
        }
        avgPriceBroughtTimesCount = renderBoughtTimesBasedOnValue(data.boughtTimes);
        if (onlyBoughtPrice) {
          return avgPriceBroughtTimes + avgPriceBroughtTimesCount;
        }
        if (onlySoldPrice) {
          return currentPrice;
        }
        return (
          avgPriceBroughtTimes +
          avgPriceBroughtTimesCount +
          '<br><span class="sold-price">' +
          currentPrice +
          '</span>' +
          getBoughtTimesHTML(data.boughtTimes, false)
        );
      }
      return data.avgPrice;
    };
  }

  function renderBoughtTimesBasedOnValue (boughtTimes) {
    let contentString;
    // Dont show bought times if it is zero.
    // To keep the alignment properly, we need to get invisible values if the boughttimes is equal to zero.
    if (+boughtTimes !== 0) {
      contentString = getBoughtTimesHTML(boughtTimes, true);
    } else {
      contentString = getBoughtTimesHTML(boughtTimes, false);
    }
    return contentString;
  }

  function getBoughtTimesHTML(boughtTimes, withoutInvisibleBlock) {
    var ColorIndex = boughtTimes * 0.005;
    var BGColor =
      boughtTimes <= MAX_BOUGHT_TIMES ? scale(ColorIndex).hex() : '#ffffff';
    var styleString = 'style="background-color:' + BGColor + '"';
    // Maximum bought times number count can be 2.
    // so to make one length and two length bought times aligned properly, adding invisible number before one length bought times.
    var invisibleText = boughtTimes.toString().length === 1
    ? '<span class="invisible">0</span>' : '';
    var contentString = boughtTimes;
    if (withoutInvisibleBlock) {
      contentString = invisibleText +
        '<span class="bought-badge-number" ' +
        styleString +
        '>' +
        boughtTimes +
        '</span>';
    } else {
      contentString = invisibleText +
        '<span class="bought-badge-number invisible" ' +
        styleString +
        '>' +
        boughtTimes +
        '</span>';
    }
    return contentString;
  }

  function getCurrentValAndTotalCost(onlyValue, onlyConvertedValue) {
    return function(data, type, row) {
      var currentValue = getCurrentValue(onlyValue, onlyConvertedValue)(
        data,
        type,
        row
      );
      if (type === 'display' || type === 'export') {
        var totalCost = handleTotalCst(onlyValue, onlyConvertedValue)(
          data,
          type,
          row
        );
        return (
          '<span class="current-value blue-color">' +
          currentValue +
          '</span><br><span class="bought-cost blue-color">' +
          totalCost +
          '</span>'
        );
      } else {
        return currentValue;
      }
    };
  }

  function handleBoughtSoldCstFrSalesLog(data, type) {
    var totalCst = handleOnlyTotalCstFrSalesLog(data, type);
    // Returns totalCst if the type of operation is sorting.
    if (type === sortOperation) {
      return totalCst;
    }
    if (displayTypesArr.indexOf(type) !== -1) {
      return totalCst + '<br>' + getOnlyCurrentValueForSalesLog(data, type);
    } else {
      return totalCst;
    }
  }

  function handleAvgValAndCurrentPrice(onlyCurentPrice, onlyAvgPrice) {
    return function(data, type, row) {
      var avgPrice = handleAvgPrice(data, type, row);
      if (type === 'display' || type === 'export') {
        var currentPrice = normalizeValueWithCommas(
          data.currentPrice,
          type,
          '',
          '',
          '',
          dtCache.currentMarketPrecision
        );
        if (onlyCurentPrice) {
          return currentPrice;
        }
        if (onlyAvgPrice) {
          return avgPrice;
        }
        return currentPrice + '<br>' + avgPrice;
      } else {
        return avgPrice;
      }
    };
  }

  function currentMarketPrecision(value, type, row) {
    return normalizeValueWithCommas(
      value,
      type,
      row,
      '',
      '',
      dtCache.currentMarketPrecision
    );
  }

  /**
   * This will add comma after three digits to the passed value parameter
   * and doesn't add comma number after decimals.
   * @param {*} value - Value to be converted.
   * @param {*} type  - It adds precision and comma only if type is 'display' or 'filter'.
   * @param {*} row
   * @param {*} set
   * @param {*} noPrecision - It will add precision to the given value if noPrecision parameter value is false value or undefined.
   * @param {*} precision - It will add passed parameter precision(if precision parameter is not passed then it adds default precision).
   */
  function normalizeValueWithCommas(
    value,
    type,
    row,
    set,
    noPrecision,
    precision
  ) {
    precision = precision || PBConstants.DEFAULT_PRECISION;
    if (!noPrecision) {
      value = checkAndAddPrecision(value, type, precision);
    }
    // Comma should be added only for type display and filter.
    if (displayTypesArr.indexOf(type) === -1) {
      return value;
    }

    value = value.toString();
    var valueArr = value.split('.');
    // Don't add comma for number after decimal.
    if (valueArr[1]) {
      valueArr[0] = valueArr[0].replace(/(\d)(?=(\d\d\d)+(?!\d))/g, '$1,');
      value = valueArr.join('.');
    } else {
      value = value.replace(/(\d)(?=(\d\d\d)+(?!\d))/g, '$1,');
    }
    return value;
  }

  function setDataCache(cache) {
    if (cache) {
      dtCache = cache;
    }
  }

  function handleBoughtSoldCstCrncy(data, type) {
    // Returns the first value directly if the operation is sort.
    if (type === sortOperation) {
      return handleOnlyTotalCstCurrencyValFrSalesLog(data, type);
    }
    return (
      handleOnlyTotalCstCurrencyValFrSalesLog(data, type) +
      '<br><span class="sold-value">' +
      getOnlyCurrCurrencyValForSalesLog(data, type) +
      '</span>'
    );
  }

  function hideCurrencyValues(dataTable, hideColumnsArr) {
    var showCurrencyColumns = true;
    // If market is USDT and currency is USD then hide the currency columns.
    if (
      dtCache.market === PBConstants.HIDE_PROFIT_MARKET_AND_ESTIMATED_USD &&
      dtCache.currency === PBConstants.BASE_CURRENCY
    ) {
      showCurrencyColumns = false;
    }
    for (var i = 0; i < hideColumnsArr.length; i++) {
      dataTable.column(hideColumnsArr[i]).visible(showCurrencyColumns);
    }
    // After changing visibility, add current currency and market as header values.
    $(document).trigger('evt.add-market-value');
  }

  function handleOrderBookProfit(data, type) {
    // TODO: Check why we are receiving type as a type.
    if (type === sortOperation || type === 'type') {
      return handleMoney(data.profit, type);
    }
    var profit = handleMoney(data.profit, type);
    var orderbookProfit = +data.orderbookProfit > 0 ? data.orderbookProfit : '';
    orderbookProfit = handleMoney(orderbookProfit, type);
    var profitClass = +profit < PBConstants.PROFIT_GREEN ? 'danger' : 'success';
    var profitString =
      '<span class="badge badge-' +
      profitClass +
      '">' +
      profit +
      '</span><br><span class="badge badge-success order-profit">' +
      orderbookProfit +
      '</span>';
    profitString = replaceBreakTag(profitString, type);
    return profitString;
  }

   // Reversing buy/sell button when shorting is true.
   function reverseBuySellButton() {
    let shorting = localStorage.getItem(NormalizerService.generateLocalStorageKey('shorting'));
    if (!JSON.parse(shorting)) {
      return;
    } else {
      var $buySellBtns = $('.buy-or-sell-btn');
      for (var i = 0; i < $buySellBtns.length; i++) {
        var currBtnType = $buySellBtns.eq(i).attr('data-buy-or-sell');
        var swappedType = currBtnType === PBConstants.BUY ? PBConstants.SELL : PBConstants.BUY;
        $buySellBtns.eq(i).attr('data-buy-or-sell', swappedType);
        $buySellBtns.eq(i).html(swappedType.toUpperCase());
        if (currBtnType === PBConstants.BUY) {
          $buySellBtns.eq(i).removeClass(PBConstants.POSITIVE_BADGE).addClass(PBConstants.NEGATIVE_BADGE);
        } else {
          $buySellBtns.eq(i).removeClass(PBConstants.NEGATIVE_BADGE).addClass(PBConstants.POSITIVE_BADGE);
        }
      }
    }
  }

  function addBuySellBtns() {
    // To select only td inside datatable.
    // In mobile resolutions, datatable will have fixed column and in that case
    // we have to select datatable inside dataTables_scrollBody div.
    var tableElement = document.querySelector('.dataTables_scrollBody .dataTable') || document.querySelector('.dataTable');
    var tableID = tableElement.id;
    var tds = tableElement.getElementsByTagName('td');
    for (var i = 0; i < tds.length; i++) {
      var outerDiv = document.createElement('DIV');
      outerDiv.classList = 'outerDiv';
      var elP = document.createElement('p');
      tds[i].appendChild(elP);
      outerDiv.style.bottom = '5px';

      var buySellBadge = document.createElement('SPAN');
      buySellBadge.style.cursor = 'pointer';
      if (tableID === "dtPendingLogs" && tds[i].classList.contains('market')) {
        outerDiv.style.right = '5px';
        var textnode = document.createTextNode('CANCEL');
        buySellBadge.appendChild(textnode);
        buySellBadge.classList =
          'badge badge-outline badge-danger buy-or-sell-btn cancel-pending hide';
        buySellBadge.dataset.pair = tds[i].parentNode.querySelector(
          '.market'
        ).childNodes[0].innerHTML;
        buySellBadge.dataset.orderNumber = tds[i].parentNode.querySelector(
          '.market'
        ).childNodes[0].dataset.order;
        buySellBadge.dataset.buyOrSell = 'cancel';
      } else
      if (tableID !== "dtPendingLogs" && tds[i].classList.contains('sell-strategy')) {
        outerDiv.style.right = '5px';
        var textnode = document.createTextNode('SELL');
        buySellBadge.appendChild(textnode);
        buySellBadge.classList =
          'badge badge-outline badge-danger buy-or-sell-btn hide';
        buySellBadge.dataset.pair = tds[i].parentNode.querySelector(
          '.market'
        ).childNodes[0].innerHTML;
        buySellBadge.dataset.buyOrSell = 'sell';
      } else if (tableID !== "dtPendingLogs" && tds[i].classList.contains('buy-strategy')) {
        var textnode = document.createTextNode('BUY');
        buySellBadge.appendChild(textnode);
        buySellBadge.classList =
          'badge badge-outline badge-success buy-or-sell-btn hide';
        buySellBadge.dataset.pair = tds[i].parentNode.querySelector(
          '.market'
        ).childNodes[0].innerHTML;
        buySellBadge.dataset.buyOrSell = 'buy';
      }
      showBuySellButtons() === 'true'
        ? buySellBadge.classList.remove('hide')
        : buySellBadge.classList.add('hide');
      outerDiv.appendChild(buySellBadge);
      tds[i].appendChild(outerDiv);
    }
    reverseBuySellButton();
  }


  /*
 * Returns actual rendering function for date column of DCA log datatable.
 * onlyLastDate boolean is used when we need only last bought date data.
 * onlyFirstDate boolean is used when we need only first bought date data.
 * onlyDate returns only date from object which is needed for excel export.
 * onlyTime returns only time from object which is needed for excel export.
 * onlyDiff returns difference in days betweeem current time and bought time.
 */
  function renderDateForDCA(
    parentProperty,
    onlyLastDate,
    onlyFirstDate,
    onlyDate,
    onlyTime,
    onlyDiff
  ) {
    return function cbDateHandler(row, type) {
      if (onlyLastDate) {
        return DataTableHelper.dateHandler(
          'lastBoughtDate',
          parentProperty,
          onlyDate,
          onlyTime,
          onlyDiff
        )(row, type);
      } else if (onlyFirstDate) {
        return DataTableHelper.dateHandler(
          'firstBoughtDate',
          parentProperty,
          onlyDate,
          onlyTime,
          onlyDiff
        )(row, type);
      }
      let date1 = DataTableHelper.dateHandler(
        'lastBoughtDate',
        parentProperty,
        onlyDate,
        onlyTime,
        onlyDiff
      )(row, type);
      let date2 = DataTableHelper.dateHandler(
        'firstBoughtDate',
        parentProperty,
        onlyDate,
        onlyTime,
        onlyDiff
      )(row, type);
      var combinedDate = date1 + "<div class=\"secondary-date-text\">" + date2 + "</div>";;
      return combinedDate;
    };
  }

  /*
* Renders seperate date column which will be only displayed in excel sheet.
* The columns created are first date, first time, first time since, last date, last time
* and last time since.
*/
  function getExportDateColumnsForDCA(columns, index, parentProperty) {
    let exportDateColumns = [
      {
        title: i18next.t('dataTableSection.lastDate.colName'),
        data: renderDateForDCA(parentProperty, true, false, true),
        visible: false
      },
      {
        title: i18next.t('dataTableSection.lastTime.colName'),
        data: renderDateForDCA(parentProperty, true, false, false, true),
        visible: false
      },
      {
        title: i18next.t('dataTableSection.lastTimeSince.colName'),
        data: renderDateForDCA(parentProperty, true, false, false, false, true),
        visible: false
      },
      {
        title: i18next.t('dataTableSection.firstDate.colName'),
        data: renderDateForDCA(parentProperty, false, true, true),
        visible: false
      },
      {
        title: i18next.t('dataTableSection.firstTime.colName'),
        data: renderDateForDCA(parentProperty, false, true, false, true),
        visible: false
      },
      {
        title: i18next.t('dataTableSection.firstTimeSince.colName'),
        data: renderDateForDCA(parentProperty, false, true, false, false, true),
        visible: false
      }
    ];

    for (let i = 0; i < exportDateColumns.length; i += 1) {
      columns.splice(index + i, 0, exportDateColumns[i]);
    }
    return columns;
  }

  function renderDataIfPresent(data) {
    // To return data only when it is defined or else returns default value.
    if (typeof data === 'undefined') {
      return '';
    }
    return data;
  }

  function handleBoughtPrice() {
    // To return function which renders normalized bought price.
    return function cbHandleBoughtPrice(data, type) {
      let currentPrice = normalizeValueWithCommas(
        data.currentPrice,
        type,
        '',
        '',
        '',
        dtCache.currentMarketPrecision
      );
      let contentString;
      if (type === 'export') {
        // Showing bought times in bracket in excel sheet.
        contentString = '<span>(' + data.boughtTimes + ')</span>';
      } else if (type === 'display') {
        // We are not displaying bought times when bought times is zero.
        contentString = renderBoughtTimesBasedOnValue(data.boughtTimes);
      }
      currentPrice += contentString;
      return currentPrice;
    };
  }

  function renderStrategyForBuyLog(data, type) {
    var strategy = '';
    var positiveCls = '';

    // Loop over strategies array.
    for (var i = 0; i < data.length; i++) {
      var currentStrategy = data[i];
      data[i] = checkAnyValueExists(data[i]);
      strategy += currentStrategy.name;
      if (dtCache.displayAdvancedStats && currentStrategy.positive && currentStrategy.positive !== 'false') {
        positiveCls = PBConstants.POSITIVE_CLASS;
        strategy +=
          '<span class="' +
          positiveCls +
          '"> (' +
          currentStrategy.positive.toString() +
          ')</span><br>';
      } else {
        positiveCls = PBConstants.NEGATIVE_CLASS;
        strategy += '<br>';
      }
    }
    strategy = replaceBreakTag(strategy, type);
    return strategy;
  }

  function getProfitForSalesLog(row) {
    if (+row.orderbookProfit === 0) {
      return row.profit;
    }
    return row.orderbookProfit;
  }

  function renderDateForDailyProfit() {
    // Rendering date for daily profits datatable.
    return function cdRenderDailyProfitDate(row) {
      var res = row.date.split("-");
      var month = monthNames[res[1] - 1];
      return month + ' ' + res[0];
    };
  }

  function handleProfitForDailySection(data, type) {
    // To element containing data got Profits in daily profit datatable.
    var profit = checkAndAddPrecision(data, type, 8);
    var currency = dtCache.currency;
    var currencyPrice = convertToCurrentCurrency(profit, type);
    var profitClass = +profit < PBConstants.PROFIT_GREEN ? 'danger' : 'success';
    return  '<span class="text-' + profitClass + '">' + profit + ' (' + currency + ' ' + currencyPrice + ')' + '</span>';
  }

  function renderProfitPercentage(data, type) {
    // To render profit percentage for daily profits table in pro tools section.
    var profitPerc;
    if (typeof data === 'undefined') {
      profitPerc = 0;
    } else {
      profitPerc = data;
    }
    profitPerc = checkAndAddPrecision(profitPerc, type, 2);
    var percClass = +profitPerc < PBConstants.PROFIT_GREEN ? 'danger' : 'success';
    return '<span class="text-' + percClass + '">' + profitPerc + '%</span>';
  }

  // public methods
  return {
    addBuySellBtns: addBuySellBtns,
    dateHandler: dateHandler,
    renderMarketCol: renderMarketCol,
    handleMoney: handleMoney,
    setIndex: setIndex,
    getDataTableStructure: getDataTableStructure,
    getCurrentValue: getCurrentValue,
    renderStrategy: renderStrategy,
    renderSellStrategy: renderSellStrategy,
    renderBuyStrategy: renderBuyStrategy,
    renderVolume: renderVolume,
    addPrecision: addPrecision,
    handleProfit: handleProfit,
    handleProfitText: handleProfitText,
    reBindData: reBindData,
    getProfitBTCForSalesLog: getProfitBTCForSalesLog,
    renderDCAEntryAndEntryLimit: renderDCAEntryAndEntryLimit,
    handleAvgPrice: handleAvgPrice,
    handleTotalAmnt: handleTotalAmnt,
    handleTotalCst: handleTotalCst,
    handleCurrentBB: handleCurrentBB,
    renderPercentageChange: renderPercentageChange,
    handleTotalCstFrSalesLog: handleTotalCstFrSalesLog,
    handleAvgPriceDcaLog: handleAvgPriceDcaLog,
    handleAvgPriceBoughtTimes: handleAvgPriceBoughtTimes,
    getCurrentValueForSalesLog: getCurrentValueForSalesLog,
    setDataCache: setDataCache,
    getCurrentValAndTotalCost: getCurrentValAndTotalCost,
    handleAvgValAndCurrentPrice: handleAvgValAndCurrentPrice,
    getCurrentValueBsdOnfee: getCurrentValueBsdOnfee,
    getCurrentTrendprofit: getCurrentTrendprofit,
    addButtons: addButtons,
    getExcelColumns: getExcelColumns,
    normalizeValueWithCommas: normalizeValueWithCommas,
    handleBoughtSoldCstFrSalesLog: handleBoughtSoldCstFrSalesLog,
    handleBoughtSoldCstCrncy: handleBoughtSoldCstCrncy,
    getCurrentDateForFileName: getCurrentDateForFileName,
    getExportDateColumns: getExportDateColumns,
    hideCurrencyValues: hideCurrencyValues,
    currentMarketPrecision: currentMarketPrecision,
    handleTrigger: handleTrigger,
    handleStratEntryVal: handleStratEntryVal,
    handlePBEntryLimit: handlePBEntryLimit,
    handleStratCurrentVal: handleStratCurrentVal,
    removeTrailingZeros: removeTrailingZeros,
    checkAllValAndHandleStratCurVal: checkAllValAndHandleStratCurVal,
    checkAllValAndHandleStratEntryVal: checkAllValAndHandleStratEntryVal,
    renderCombinedMarketCol: renderCombinedMarketCol,
    handleOrderBookProfit: handleOrderBookProfit,
    getCurrentPaginationNumber: getCurrentPaginationNumber,
    setDTPagination: setDTPagination,
    renderDateForDCA: renderDateForDCA,
    getExportDateColumnsForDCA: getExportDateColumnsForDCA,
    handleBoughtPrice: handleBoughtPrice,
    renderDataIfPresent: renderDataIfPresent,
    renderStrategyForBuyLog: renderStrategyForBuyLog,
    handleBuyLogBoughtAmnt: handleBuyLogBoughtAmnt,
    isFixedColumnsResolution: isFixedColumnsResolution,
    getProfitForSalesLog: getProfitForSalesLog,
    renderDateForDailyProfit: renderDateForDailyProfit,
    handleProfitForDailySection: handleProfitForDailySection,
    renderProfitPercentage: renderProfitPercentage
  };
})();
